import 'package:cl_datahub/cl_datahub.dart';

abstract class BaseService {
  final ConfigPath? configPath;

  BaseService([String? path])
      : configPath = path == null ? null : ConfigPath(path);

  /// Fetches a configuration value from [ConfigService].
  T config<T>(String path, {T? defaultValue}) {
    final relative = ConfigPath(path);
    final absolute = configPath?.join(relative) ?? relative;
    return resolve<ConfigService>()
        .fetch<T>(absolute, defaultValue: defaultValue);
  }

  /// Fetches a configuration value from [ConfigService] and parse it into the
  /// TransferObject.
  T configObject<T extends TransferObjectBase>(
      String path, TransferBean<T> bean) {
    final relative = ConfigPath(path);
    final absolute = configPath?.join(relative) ?? relative;
    return resolve<ConfigService>().fetchObject<T>(absolute, bean);
  }

  Future<void> initialize();

  Future<void> shutdown();
}
